"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CorePico = void 0;
const _ = require("lodash");
const pico_framework_1 = require("pico-framework");
const makeKrlCtxKrlModule_1 = require("./makeKrlCtxKrlModule");
class CorePico {
    constructor(core) {
        this.core = core;
        this.dependencies = {};
    }
    async use(krlCtx, usesRid, alias, configure) {
        var _a;
        const rsCtx = krlCtx.rsCtx;
        const picoId = rsCtx.pico().id;
        let pfPico;
        try {
            pfPico = this.core.picoFramework.getPico(picoId);
        }
        catch (err) {
            throw new Error("PicoFramework not yet setup");
        }
        const usesRidConfig = ((_a = pfPico.rulesets[usesRid]) === null || _a === void 0 ? void 0 : _a.config) || {};
        const ruleset = this.core.rsRegistry.getCached(usesRidConfig.url || "");
        if (!ruleset) {
            throw new Error(`Module not found: ${usesRid}`);
        }
        const rsI = await ruleset.ruleset.init((0, pico_framework_1.createRulesetContext)(this.core.picoFramework, pfPico, {
            rid: ruleset.rid,
            config: Object.assign(Object.assign({}, usesRidConfig), { _krl_module_config: configure }),
        }), (rsCtx2) => (0, makeKrlCtxKrlModule_1.makeKrlCtxKrlModule)(this.core, krlCtx, rsCtx2));
        const module = rsI.provides || {};
        if (!alias) {
            alias = usesRid;
        }
        if (!this.dependencies[rsCtx.ruleset.rid]) {
            this.dependencies[rsCtx.ruleset.rid] = {
                krlCtx,
                uses: {},
            };
        }
        if (!this.dependencies[rsCtx.ruleset.rid].uses[usesRid]) {
            this.dependencies[rsCtx.ruleset.rid].uses[usesRid] = {};
        }
        this.dependencies[rsCtx.ruleset.rid].uses[usesRid][alias] = {
            configure,
            url: ruleset.url,
            hash: ruleset.hash,
            module,
        };
    }
    configure(rsCtx, name, dflt) {
        const config = rsCtx.ruleset.config;
        if (_.has(config, ["_krl_module_config", name])) {
            return config._krl_module_config[name];
        }
        return dflt;
    }
    getModule(userRid, domain) {
        if (this.dependencies[userRid]) {
            for (const rm of Object.values(this.dependencies[userRid].uses)) {
                if (rm[domain]) {
                    return rm[domain].module;
                }
            }
        }
        return this.core.modules[domain] || null;
    }
    whoUses(rid) {
        const userRids = [];
        for (const userRid of Object.keys(this.dependencies)) {
            if (this.dependencies[userRid].uses[rid]) {
                userRids.push(userRid);
            }
        }
        return userRids;
    }
    unUse(rid) {
        delete this.dependencies[rid];
    }
    onRulesetLoaded(crs) {
        const rid = crs.ruleset.rid;
        for (const userRid of Object.keys(this.dependencies)) {
            const { uses, krlCtx } = this.dependencies[userRid];
            if (uses[rid]) {
                for (const alias of Object.keys(uses[rid])) {
                    const rsm = uses[rid][alias];
                    if (rsm.url === crs.url && rsm.hash !== crs.hash) {
                        this.use(krlCtx, rid, alias, rsm.configure).catch((error) => {
                            krlCtx.log.error("Failed to update dependent ruleset", {
                                userRid,
                                url: crs.url,
                                hash: crs.hash,
                                error,
                            });
                        });
                    }
                }
            }
        }
    }
}
exports.CorePico = CorePico;
//# sourceMappingURL=CorePico.js.map