"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeKrlCtx = void 0;
const krl_stdlib_1 = require("krl-stdlib");
function makeKrlCtx(core, rsCtx) {
    const pico = rsCtx.pico();
    const picoId = pico.id;
    const logCtxBase = { picoId, rid: rsCtx.ruleset.rid };
    let log = core.log.child(logCtxBase);
    let corePico = core.addPico(picoId);
    let currentEvent = null;
    let currentQuery = null;
    let currentRuleName = null;
    let directives = [];
    const krlCtx = {
        rsCtx,
        ///////////////////////////////////////////////////////////////////////////
        // logging
        log,
        getPicoLogs() {
            return core.getPicoLogs(picoId);
        },
        ///////////////////////////////////////////////////////////////////////////
        // current event/query
        getEvent() {
            return currentEvent;
        },
        setEvent(event) {
            krlCtx.log = log = core.log.child(event ? Object.assign(Object.assign({}, logCtxBase), { txnId: event.eid }) : logCtxBase);
            currentEvent = event;
        },
        getCurrentRuleName() {
            return currentRuleName;
        },
        setCurrentRuleName(ruleName) {
            currentRuleName = ruleName;
        },
        getQuery() {
            return currentQuery;
        },
        setQuery(query) {
            krlCtx.log = log = core.log.child(query ? Object.assign(Object.assign({}, logCtxBase), { txnId: query.qid }) : logCtxBase);
            currentQuery = query;
        },
        ///////////////////////////////////////////////////////////////////////////
        // modules
        module(domain) {
            return corePico.getModule(rsCtx.ruleset.rid, domain);
        },
        configure(name, dflt) {
            return corePico.configure(rsCtx, name, dflt);
        },
        async useModule(rid, alias, configure) {
            await corePico.use(krlCtx, rid, alias, configure);
        },
        ///////////////////////////////////////////////////////////////////////////
        // directives
        addDirective(name, options) {
            const directive = {
                type: "directive",
                name,
                options: options || {},
                meta: {
                    rid: rsCtx.ruleset.rid,
                    rule_name: currentRuleName,
                    txnId: (currentEvent === null || currentEvent === void 0 ? void 0 : currentEvent.eid) || null,
                },
            };
            directives.push(directive);
            return directive;
        },
        drainDirectives() {
            const tmp = directives;
            directives = [];
            return tmp;
        },
        ///////////////////////////////////////////////////////////////////////////
        // compiler lib
        krl: krl_stdlib_1.krl,
    };
    return krlCtx;
}
exports.makeKrlCtx = makeKrlCtx;
//# sourceMappingURL=makeKrlCtx.js.map