"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeKrlCtxKrlModule = void 0;
const krl_stdlib_1 = require("krl-stdlib");
function makeKrlCtxKrlModule(core, host, rsCtx) {
    const pico = rsCtx.pico();
    const picoId = pico.id;
    let corePico = core.addPico(picoId);
    const krlCtx = {
        rsCtx,
        ///////////////////////////////////////////////////////////////////////////
        // logging
        get log() {
            return host.log.child({ rid: rsCtx.ruleset.rid });
        },
        getPicoLogs() {
            return host.getPicoLogs();
        },
        ///////////////////////////////////////////////////////////////////////////
        // current event/query is not applicable for krl modules
        getEvent: () => null,
        setEvent(event) { },
        getCurrentRuleName: () => null,
        setCurrentRuleName(ruleName) { },
        getQuery: () => null,
        setQuery(query) { },
        ///////////////////////////////////////////////////////////////////////////
        // modules
        module(domain) {
            return corePico.getModule(rsCtx.ruleset.rid, domain);
        },
        configure(name, dflt) {
            return corePico.configure(rsCtx, name, dflt);
        },
        async useModule(rid, alias, configure) {
            await corePico.use(krlCtx, rid, alias, configure);
        },
        ///////////////////////////////////////////////////////////////////////////
        // directives
        addDirective(name, options) {
            // if the module provides a defaction that uses send_directive, append to the host's directives
            return host.addDirective(name, options);
        },
        drainDirectives: () => [],
        ///////////////////////////////////////////////////////////////////////////
        // compiler lib
        krl: // not used since krl modules are not installed
        krl_stdlib_1.krl,
    };
    return krlCtx;
}
exports.makeKrlCtxKrlModule = makeKrlCtxKrlModule;
//# sourceMappingURL=makeKrlCtxKrlModule.js.map