"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cross_fetch_1 = require("cross-fetch");
const krl_stdlib_1 = require("krl-stdlib");
function eventProperty(fn) {
    return krl_stdlib_1.krl.Property(function () {
        let event = this.getEvent();
        if (event) {
            return fn(event);
        }
        return null;
    });
}
const event = {
    eci: eventProperty((event) => event.eci),
    domain: eventProperty((event) => event.domain),
    name: eventProperty((event) => event.name),
    attrs: eventProperty((event) => event.data.attrs),
    attr: krl_stdlib_1.krl.Function(["name"], function (name) {
        let event = this.getEvent();
        if (event) {
            return event.data.attrs[name];
        }
        return null;
    }),
    eid: eventProperty((event) => event.eid),
    time: eventProperty((event) => event.time),
    send: krl_stdlib_1.krl.Action(["options", "host"], async function event(options, host) {
        let eci = options.eci;
        let domain = options.domain;
        let name = options.name || options.type;
        let attrs = options.attrs;
        if (!eci) {
            if (options.sub) {
                eci = options.sub.Tx;
                host = options.sub.Tx_host;
            }
            else if (options.did) {
                await this.useModule("io.picolabs.did-o", "didx");
                await this.krl.assertFunction(this.module("didx")["sendEvent"])(this, [options.did, {
                        "domain": domain,
                        "name": name,
                        "attrs": attrs
                    }]);
                return;
            }
        }
        if (!krl_stdlib_1.krl.isString(eci)) {
            throw new TypeError("eci was " +
                krl_stdlib_1.krl.toString(eci) +
                " instead of a string");
        }
        if (host) {
            const url = `${host}/c/${eci}/event/${domain}/${name}`;
            (0, cross_fetch_1.default)(url, {
                method: "POST",
                headers: { "content-type": "application/json" },
                body: krl_stdlib_1.krl.encode(attrs),
            }).catch((err) => {
                this.log.error(err + ""); // TODO better handling
            });
            return;
        }
        // fire-n-forget event not eventWait
        const eid = await this.rsCtx.event({
            eci,
            domain,
            name,
            data: { attrs },
            time: 0,
        }).catch((err) => {
            this.log.error(err + ""); // TODO better handling
        });
        return eid;
    }),
};
exports.default = event;
//# sourceMappingURL=event.js.map