"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
const krl_stdlib_1 = require("krl-stdlib");
const _ = require("lodash");
const hashAlgorithms = Object.freeze(crypto.getHashes());
const math = {
    base64encode: krl_stdlib_1.krl.Function(["str", "str"], function (str, encoding = "utf8") {
        str = krl_stdlib_1.krl.toString(str);
        return Buffer.from(str, encoding).toString("base64");
    }),
    base64decode: krl_stdlib_1.krl.Function(["str", "str"], function (str, encoding = "utf8") {
        str = krl_stdlib_1.krl.toString(str);
        return Buffer.from(str, "base64").toString(encoding);
    }),
    hashAlgorithms: krl_stdlib_1.krl.Property(() => {
        return hashAlgorithms;
    }),
    hashFunctions: krl_stdlib_1.krl.Function([
    // DEPRECATED
    ], function () {
        this.log.warn("math:hashFunctions() is DEPRECATED use math:hashAlgorithms instead");
        return hashAlgorithms;
    }),
    hash: krl_stdlib_1.krl.Function(["algorithm", "str", "encoding"], function (algorithm, str, encoding) {
        if (!_.includes(hashAlgorithms, algorithm)) {
            if (krl_stdlib_1.krl.isString(algorithm)) {
                throw new Error("math:hash doesn't recognize the hash algorithm " + algorithm);
            }
            else {
                throw new TypeError("math:hash was given " +
                    krl_stdlib_1.krl.toString(algorithm) +
                    " instead of a algorithm string");
            }
        }
        encoding = encoding || "hex";
        if (!_.includes(["hex", "base64"], encoding)) {
            throw new Error('math:hash encoding must be "hex" or "base64" but was ' + encoding);
        }
        str = krl_stdlib_1.krl.toString(str);
        var hash = crypto.createHash(algorithm);
        hash.update(str);
        return hash.digest(encoding);
    }),
    hmac: krl_stdlib_1.krl.Function(["algorithm", "key", "message", "encoding"], function (algorithm, key, message, encoding) {
        if (!krl_stdlib_1.krl.isString(key)) {
            throw new Error("math:hmac needs a key string");
        }
        if (!_.includes(hashAlgorithms, algorithm)) {
            if (krl_stdlib_1.krl.isString(algorithm)) {
                throw new Error("math:hmac doesn't recognize the hash algorithm " + algorithm);
            }
            else {
                throw new TypeError("math:hmac was given " +
                    krl_stdlib_1.krl.toString(algorithm) +
                    " instead of a algorithm string");
            }
        }
        encoding = encoding || "hex";
        if (!_.includes(["hex", "base64"], encoding)) {
            throw new Error('math:hmac encoding must be "hex" or "base64" but was ' + encoding);
        }
        key = krl_stdlib_1.krl.toString(key);
        message = krl_stdlib_1.krl.toString(message);
        var hmac = crypto.createHmac(algorithm, key);
        hmac.update(message);
        return hmac.digest(encoding);
    }),
    abs: krl_stdlib_1.krl.Function(["number"], function (number) {
        return Math.abs(number);
    }),
    ceiling: krl_stdlib_1.krl.Function(["number", "precision"], function (number, precision) {
        return _.ceil(number, precision);
    }),
    floor: krl_stdlib_1.krl.Function(["number", "precision"], function (number, precision) {
        return _.floor(number, precision);
    }),
    int: krl_stdlib_1.krl.Function(["number"], function (number) {
        return number >= 0 ? _.floor(number, 0) : _.ceil(number, 0);
    }),
    round: krl_stdlib_1.krl.Function(["number", "precision"], function (number, precision) {
        return _.round(number, precision);
    }),
};
exports.default = math;
//# sourceMappingURL=math.js.map